require 'docker-api'
require 'active_support'

class Docker::Swarm::Service
  attr_reader :hash

  def initialize(swarm, hash)
    @swarm = swarm
    @hash = hash
  end

  def name()
    @hash['Spec']['Name']
  end

  def id()
    return @hash['ID']
  end

  def reload()
    s = @swarm.find_service(id())
    @hash = s.hash
    return self
  end

  def network_ids
    network_ids = []
    if (@hash['Endpoint']['VirtualIPs'])
      @hash['Endpoint']['VirtualIPs'].each do |network_info|
        network_ids << network_info['NetworkID']
      end
    end
    return network_ids
  end

  def remove(opts = {})
    query = {}
    @swarm.connection.delete("/services/#{self.id}", query, :body => opts.to_json)
  end

  def update(options = {})
    specs = @hash['Spec'].deep_merge(options)
    query = {}
    version = @hash['Version']['Index']
    response = @swarm.connection.post("/services/#{self.id}/update?version=#{version}", query, :body => specs.to_json)
  end

  def tasks
    service_tasks = []
    @swarm.tasks.each do |task|
      service_tasks << task if (task.service_id == self.id)
    end
    return service_tasks
  end

  def restart
    options = {}
    options['TaskTemplate'] = {'ForceUpdate' => 1}
    update(options)
  end

  def scale(count)
    @hash['Spec']['Mode']['Replicated']['Replicas'] = count
    self.update(@hash['Spec'])
  end

  def replicas
    @hash['Spec']['Mode']['Replicated']['Replicas']
  end

  def tasks_with_status(status)
    running = []
    tasks.each do |task|
      if (task.status == status)
        running << task
      end
    end
    return running
  end


  def tasks_running
    return tasks_with_status(:running)
  end

  def tasks_starting
    return tasks_with_status(:starting)
  end

  def self.DEFAULT_OPTIONS
    default_service_create_options = {
        "Name" => "<<Required>>",
        "TaskTemplate" => {
          "ContainerSpec" => {
            "Image" => "<<Required>>",
            "Mounts" => [],
            "User" => "root"
          },
          "Env" => [],
          "LogDriver" => {
            "Name" => "json-file",
            "Options" => {
              "max-file" => "3",
              "max-size" => "10M"
            }
          },
          "Placement" => {},
          "Resources" => {
            "Limits" => {
              "MemoryBytes" => 104857600
            },
            "Reservations" => {
#              "NanoCPUs" => ?
#              MemoryBytes =>
           }
          },
          "RestartPolicy" => {
            "Condition" => "on-failure",
            "Delay" => 1,
            "MaxAttempts" => 3
          }
        }, # End of TaskTemplate
        "Mode" => {
          "Replicated" => {
            "Replicas" => 1
          }
        },
        "UpdateConfig" => {
          "Delay" => 2,
          "Parallelism" => 2,
          "FailureAction" => "pause"
        },
        "EndpointSpec" => {
          "Ports" => [
            {
    #          "Protocol" => "http",
    #          "PublishedPort" => 2881,
    #          "TargetPort" => 2881
            }
          ]
        },
        "Labels" => {
          "foo" => "bar"
        }
      }
    return default_service_create_options
  end
end
